From 526409e76419b387d9d4a40e94881231f256fcc8 Mon Sep 17 00:00:00 2001
From: Pavel Borisov <pashkin.elfe@gmail.com>
Date: Thu, 25 Apr 2024 14:20:45 +0400
Subject: [PATCH v1 3/5] Amcheck: Don't load rightpage into BtreeCheckState

For cross-page unique constraint check use a local variable in the
similar way as implemented in bt_right_page_check_scankey().

Reported-by: Peter Geoghegan
---
 contrib/amcheck/verify_nbtree.c | 12 ++++++++----
 1 file changed, 8 insertions(+), 4 deletions(-)

diff --git a/contrib/amcheck/verify_nbtree.c b/contrib/amcheck/verify_nbtree.c
index 66f2e619a8..d6f70206db 100644
--- a/contrib/amcheck/verify_nbtree.c
+++ b/contrib/amcheck/verify_nbtree.c
@@ -1890,23 +1890,27 @@ bt_target_page_check(BtreeCheckState *state)
 				/* The first key on the next page is the same */
 				if (_bt_compare(state->rel, rightkey, state->target, max) == 0 && !rightkey->anynullkeys)
 				{
+					Page	rightpage;
+
 					if (!unique_checked)
 						bt_entry_unique_check(state, itup, state->targetblock, offset, &lVis);
 
 					elog(DEBUG2, "cross page equal keys");
-					state->target = palloc_btree_page(state,
+					rightpage = palloc_btree_page(state,
 													  rightblock_number);
-					topaque = BTPageGetOpaque(state->target);
+					topaque = BTPageGetOpaque(rightpage);
 
 					if (P_IGNORE(topaque) || !P_ISLEAF(topaque))
 						break;
 
 					itemid = PageGetItemIdCareful(state, rightblock_number,
-												  state->target,
+												  rightpage,
 												  rightfirstoffset);
-					itup = (IndexTuple) PageGetItem(state->target, itemid);
+					itup = (IndexTuple) PageGetItem(rightpage, itemid);
 
 					bt_entry_unique_check(state, itup, rightblock_number, rightfirstoffset, &lVis);
+
+					pfree(rightpage);
 				}
 			}
 		}
-- 
2.34.1

